>From 1ff0759a25fc456ff07647f97f570b75665812e6 Mon Sep 17 00:00:00 2001
From: Erwan Tulou <brezhoneg1@yahoo.fr>
Date: Tue, 21 Apr 2009 21:58:55 +0200
Subject: [PATCH] enable zooming when --no-qt-video-autoresize is set

---
 modules/gui/qt4/main_interface.cpp |   75 ++++++++++++++++++++++++++++++------
 modules/gui/qt4/main_interface.hpp |   20 +++++++++-
 2 files changed, 82 insertions(+), 13 deletions(-)

diff --git a/modules/gui/qt4/main_interface.cpp b/modules/gui/qt4/main_interface.cpp
index f209038..6d4936f 100644
--- a/modules/gui/qt4/main_interface.cpp
+++ b/modules/gui/qt4/main_interface.cpp
@@ -68,8 +68,11 @@ static int PopupMenuCB( vlc_object_t *p_this, const char *psz_variable,
                         vlc_value_t old_val, vlc_value_t new_val, void *param );
 static int IntfShowCB( vlc_object_t *p_this, const char *psz_variable,
                        vlc_value_t old_val, vlc_value_t new_val, void *param );
+static int ZoomCB( vlc_object_t *p_this, const char *psz_variable,
+                   vlc_value_t old_val, vlc_value_t new_val, void *param );
 
-MainInterface::MainInterface( intf_thread_t *_p_intf ) : QVLCMW( _p_intf )
+MainInterface::MainInterface( intf_thread_t *_p_intf )
+    : QVLCMW( _p_intf ), b_zooming( false )
 {
     /* Variables initialisation */
     // need_components_update = false;
@@ -195,6 +198,14 @@ MainInterface::MainInterface( intf_thread_t *_p_intf ) : QVLCMW( _p_intf )
                  this, updateSystrayTooltipStatus( int ) );
     }
 
+    /* tracking zoom changes to adapt even if keep_size is true  */
+    if( b_keep_size )
+    {
+        vlc_mutex_init( &zoom_lock );
+        CONNECT( THEMIM->getIM(), voutChanged( bool ),
+                 this, updateZoomCallback( bool ) );
+    }
+
     /* END CONNECTS ON IM */
 
     dialogHandler = new DialogHandler (p_intf);
@@ -309,6 +320,9 @@ MainInterface::~MainInterface()
     settings->setValue( "mainBasedSize", mainBasedSize );
     settings->setValue( "mainVideoSize", mainVideoSize );
 
+    if( b_keep_size )
+        vlc_mutex_destroy( &zoom_lock );
+
     if( bgWidget )
         settings->setValue( "backgroundSize", bgWidget->size() );
 
@@ -557,6 +571,19 @@ int MainInterface::privacyDialog( QList<ConfigControl *> *controls )
     return privacy->exec();
 }
 
+void MainInterface::updateZoomCallback( bool b_video )
+{
+    if( b_video )
+    {
+        vout_thread_t* p_vout = THEMIM->getVout();
+        if( p_vout )
+        {
+            var_AddCallback( p_vout, "zoom", ZoomCB, p_intf );
+            vlc_object_release( p_vout );
+        }
+    }
+}
+
 
 /**********************************************************************
  * Handling of sizing of the components
@@ -571,23 +598,32 @@ int MainInterface::privacyDialog( QList<ConfigControl *> *controls )
    If you think this would be better, please FIXME it...
 */
 
-QSize MainInterface::sizeHint() const
+QSize MainInterface::sizeHint()
 {
     if( b_keep_size )
     {
-        if( i_visualmode == QT_ALWAYS_VIDEO_MODE ||
-            i_visualmode == QT_MINIMAL_MODE )
+        bool b_zoom = getZooming();
+        if( !b_zoom )
         {
-                return mainVideoSize;
+            if( i_visualmode == QT_ALWAYS_VIDEO_MODE ||
+                i_visualmode == QT_MINIMAL_MODE )
+            {
+                    return mainVideoSize;
+            }
+            else
+            {
+                if( VISIBLE( bgWidget) ||
+                    ( videoIsActive && videoWidget->isVisible() )
+                  )
+                    return mainVideoSize;
+                else
+                    return mainBasedSize;
+            }
         }
         else
         {
-            if( VISIBLE( bgWidget) ||
-                ( videoIsActive && videoWidget->isVisible() )
-              )
-                return mainVideoSize;
-            else
-                return mainBasedSize;
+            /* reset zooming */
+            setZooming( false );
         }
     }
 
@@ -701,9 +737,12 @@ WId MainInterface::requestVideo( vout_thread_t *p_nvout, int *pi_x,
                                  int *pi_y, unsigned int *pi_width,
                                  unsigned int *pi_height )
 {
+    bool b_zoom = b_keep_size ? getZooming() : false;
+
     /* Request the videoWidget */
     WId ret = videoWidget->request( p_nvout,pi_x, pi_y,
-                                    pi_width, pi_height, b_keep_size );
+                                    pi_width, pi_height,
+                                    b_keep_size && !b_zoom );
     if( ret ) /* The videoWidget is available */
     {
         /* Did we have a bg ? Hide it! */
@@ -1269,3 +1308,15 @@ static int IntfShowCB( vlc_object_t *p_this, const char *psz_variable,
      return VLC_SUCCESS;
 }
 
+/*****************************************************************************
+ * ZoomCB: callback triggered by the zoom variable (vout thread)
+ *****************************************************************************/
+static int ZoomCB( vlc_object_t *p_this, const char *psz_variable,
+                   vlc_value_t old_val, vlc_value_t new_val, void *param )
+{
+    intf_thread_t *p_intf = (intf_thread_t *)param;
+    p_intf->p_sys->p_mi->setZooming( true );
+
+    return VLC_SUCCESS;
+}
+
diff --git a/modules/gui/qt4/main_interface.hpp b/modules/gui/qt4/main_interface.hpp
index e61fa9c..f396e14 100644
--- a/modules/gui/qt4/main_interface.hpp
+++ b/modules/gui/qt4/main_interface.hpp
@@ -87,7 +87,22 @@ public:
     int getControlsVisibilityStatus();
 
     /* Sizehint() */
-    virtual QSize sizeHint() const;
+    virtual QSize sizeHint();
+
+    /* tracking zooming */
+    bool getZooming( )
+    {
+        vlc_mutex_lock( &zoom_lock );
+        bool b_ret = b_zooming;
+        vlc_mutex_unlock( &zoom_lock );
+        return b_ret;
+    }
+    void setZooming( bool b_zoom )
+    {
+        vlc_mutex_lock( &zoom_lock );
+        b_zooming = b_zoom;
+        vlc_mutex_unlock( &zoom_lock );
+    }
 
 protected:
     void dropEventPlay( QDropEvent *, bool);
@@ -140,6 +155,8 @@ private:
     bool                 b_keep_size;         ///< persistent resizeable window
     QSize                mainBasedSize;       ///< based Wnd (normal mode only)
     QSize                mainVideoSize;       ///< Wnd with video (all modes)
+    bool                 b_zooming;           ///< is zooming being triggered ?
+    vlc_mutex_t          zoom_lock;           ///  lock to protect b_zooming
     int                  i_visualmode;        ///< Visual Mode
     pl_dock_e            i_pl_dock;
     bool                 isDocked() { return ( i_pl_dock != PL_UNDOCKED ); }
@@ -169,6 +186,7 @@ private slots:
     void doComponentsUpdate();
     void setName( QString );
     void setVLCWindowsTitle( QString title = "" );
+    void updateZoomCallback( bool );
 #if 0
     void visual();
 #endif
-- 
1.5.2.5

